﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Sharp1No2 : MonoBehaviour
{
    void Start()
    {
        // ステップインでそれぞれ実行してみよう

        If();
        ConditionalOperator();

        For();
        For2();

        For3NoGood();

        For3Good1();
        For3Good2();
    }

    /// <summary>
    /// if 文で条件判定する
    /// </summary>
    void If()
    {
        // boolは論理値 (true か false)
        bool t = true;
        bool f = false;

        bool and = t && f;    // true かつ false
        bool or = t || f;     // true または false

        int i = 10;
        bool b3 = i == 10;
        bool b4 = i != 10;

        // if と打ってTabキーを2回押すと、コードスニペットが挿入される
        if (i == 10)
        {
            // i == 10 の場合
            t = true;
        }
        else
        {
            // i == 10 ではない場合 (i != 10 と同じ)
            f = false;
        }

        // i == 10 と結果は同じ。if() にはboolを渡せば良い
        if (b3)
        {
            t = true;
        }
        else
        {
            f = false;
        }

        // メソッドの戻り値が bool なら、if() にメソッドを書いても良い
        if (Is10(i))
        {
            t = true;
        }
        else
        {
            f = false;
        }

        // && || で複数条件にできる (判定順序は && の後に ||)
        if (5 <= i && i <= 15 || i == 100)
        {
            // i が 5～15または100 なら trueになる
            t = true;
        }

        // このとき、 i == 100 の判定は実行されていない

        // 実験 https://try.dot.net/
        /*

        var result = T("1") && T("2") || T("3");
        //var result = T("1") || T("2") && T("3");
        //var result = F("1") || T("2") && T("3");
        //var result = F("1") || T("2") || T("3");
        Console.WriteLine(result);

        bool T(string s)
        {
            Console.WriteLine(s);
            return true;
        }

        bool F(string s)
        {
            Console.WriteLine(s);
            return false;
        }

        */
    }

    /// <summary>
    /// 受け取った引数 i が 10 なら true を返す
    /// </summary>
    /// <param name="i">判定対象の値</param>
    /// <returns>判定対象の値が10なら true、それ以外は false</returns>
    bool Is10(int i)
    {
        return i == 10;
    }

    /// <summary>
    /// 三項演算子
    /// </summary>
    void ConditionalOperator()
    {
        int hp = 5;
        int damage = 100;

        if (hp <= 10)
        {
            damage = 150;
        }

        // 1行で書ける (damageを2回書かなくて済む)
        damage = hp <= 10 ? 150 : 100;

        // このくらいなら使いやすい
        // && や || があるような複雑なときは使わない！
        damage = 9 <= hp ? 200 :
                 6 <= hp ? 100 :
                 3 <= hp ? 50 : 30;
    }

    /// <summary>
    /// for 文で繰り返し(ループ)処理する
    /// </summary>
    void For()
    {
        // for と打ってTabキーを2回押すと、コードスニペットが挿入される

        int sum = 0;
        for (int i = 0; i < 5; i++)
        {
            sum += i;
        }
        // 0 + 1 + 2 + 3 + 4 = 6

        int counter = 0;
        int value = 0;

        for (/* 初期値 */; /* ループ条件 */; /* 次のループに移る際に実行する処理 */)
        // for (;;) は無限ループになる・・・が、これは普段使わない
        {
            ++counter;

            if (counter % 2 == 0)
            {
                // 次のループに移る
                continue;
            }

            if (counter >= 10)
            {
                // ループを抜ける
                break;
            }

            value = counter;
        }
    }

    /// <summary>
    /// 二重ループ
    /// </summary>
    void For2()
    {
        // const をつけると定数として宣言される
        const int answer = 16;

        // 書き換え不能 (以下のコメントを外すと、コンパイルエラー)
        //answer = 16;

        // 二重ループ
        for (int i = 1; i <= 9; i++)
        {
            for (int j = 1; j <= 9; j++)
            {
                // 九九で答えが 16 になったら終了
                if (i * j == answer)
                {
                    // jのループを抜けるだけ
                    break;
                }

                if (i * j == answer)
                {
                    // For2()メソッド自体は終了する
                    return;
                }
            }
        }

        // ここで何か処理したい場合はどうするか？
        // returnだとここに到達しない
    }

    /// <summary>
    /// 二重ループの最後に処理を入れてみる
    /// </summary>
    void For3NoGood()
    {
        const int answer = 16;
        bool hit = false;

        for (int i = 1; i <= 9; i++)
        {
            for (int j = 1; j <= 9; j++)
            {
                if (i * j == answer)
                {
                    hit = true;
                    break;
                }
            }

            // break; 2回で抜ける
            if (hit)
            {
                break;
            }
        }

        if (hit)
        {
            // 16があったときの処理
        }
        else
        {
            // 16がなかったときの処理
        }
    }

    #region ********** 二重ループの改善版1 **********

    void For3Good1()
    {
        bool hit = false;

        for (int i = 1; i <= 9; i++)
        {
            if (For3GoodSub1(i))
            {
                hit = true;
                break;
            }
        }

        if (hit)
        {
            // 16があったときの処理
        }
        else
        {
            // 16がなかったときの処理
        }
    }

    bool For3GoodSub1(int i)
    {
        const int answer = 16;

        for (int j = 1; j <= 9; j++)
        {
            if (i * j == answer)
            {
                return true;
            }
        }

        return false;
    }

    #endregion

    #region ********** 二重ループの改善版2 **********

    void For3Good2()
    {
        if (For3GoodSub2())
        {
            // 16があったときの処理
        }
        else
        {
            // 16がなかったときの処理
        }
    }

    bool For3GoodSub2()
    {
        // 九九に 16 の答えがあるかの判定だけに専念したメソッド
        const int answer = 16;

        for (int i = 1; i <= 9; i++)
        {
            for (int j = 1; j <= 9; j++)
            {
                if (i * j == answer)
                {
                    return true;
                }
            }
        }

        return false;
    }

    #endregion
}